#include<avr/io.h>
#include"MMC_SD/MMC_SD.h"
#include"FAT/FAT.h"
#include <util/delay.h>

// control line pins
#define	LCD_CD	PC7
#define	LCD_WR	PC6
#define	LCD_RD	PC5
#define	LCD_CS	PC4
#define	LCD_RES	PC2

// macros for control lines
#define CLR_CD		PORTC &= ~(1<<LCD_CD);
#define SET_CD		PORTC |=  (1<<LCD_CD);

#define CLR_CS		PORTC &= ~(1<<LCD_CS);
#define SET_CS		PORTC |=  (1<<LCD_CS);

#define CLR_RESET	PORTC &= ~(1<<LCD_RES);
#define SET_RESET	PORTC |=  (1<<LCD_RES);

#define CLR_WR		PORTC &= ~(1 << LCD_WR);
#define SET_WR		PORTC |=  (1 << LCD_WR);
#define CLR_RD		PORTC &= ~(1 << LCD_RD);
#define SET_RD		PORTC |=  (1 << LCD_RD);

// color definitions
#define BLACK	0x0000
#define BLUE	0x001F
#define RED 	0xF800
#define GREEN	0x07E0
#define CYAN	0x07FF
#define MAGENTA	0xF81F
#define YELLOW	0xFFE0
#define WHITE	0xFFFF

#define uint8	unsigned char
#define uint16	unsigned int
#define uint32	unsigned long

unsigned char * PATH = "\\batch";

extern uint16 SectorsPerClust;
extern uint16 FirstDataSector; 
extern uint8 FAT32_Enable;

struct FileInfoStruct FileInfo;

struct direntry PictureInfo;
uint16 totalfiles = 0;

#define HDP	799		//Hsync Display period 
#define HT	1056	//Horizontal total period = (HT + 1) pixels
#define HPS	216		//Horizontal Sync Pulse Start Position = (HPS + 1) pixels
#define LPS	6		//Horizontal Display Period Start Position = LPS pixels
#define HPW	216		//Horizontal Sync Pulse Width = (HPW + 1) pixels

#define VDP	481		//Vsync Display period
#define VT	493		//Vertical Total = (VT + 1) lines
#define VPS	12		//Vertical Sync Pulse Start Position = VPS lines
#define FPS	8		//Vertical Display Period Start Position = FPS lines
#define VPW	11		//Vertical Sync Pulse Width = (VPW + 1) lines

/*************************************************/
void delay(unsigned int t)
{
	unsigned char t1;
	while(t--)
	for(t1=11;t1>0;t1--)
	{asm("nop");}
}
/*************************************************/
void write_command(unsigned int command)
{
	CLR_CD;
	SET_RD;
	CLR_WR;
	CLR_CS;

	PORTA=command;
	PORTE=command>>8;

	SET_WR;
	SET_CS;
}

/*************************************************/
void write_data(unsigned int data)
{
	SET_CD;
	SET_RD;
	CLR_WR;
	CLR_CS;

	PORTA=data;
	PORTE=data>>8;

	SET_WR;
	SET_CS;
}
/*************************************************/
//*******************************************************************
void wr_com8080_16(unsigned char data1,unsigned char data2)
{
	CLR_CD;
	CLR_WR;
	CLR_CS;
	SET_RD;

	PORTE = data1;
	PORTA = data2;

	SET_CS;
	SET_WR;
	SET_CD;
}
//*******************************************************
void wr_data8080_16(unsigned char data1,unsigned char data2)

{
	SET_CD;
	CLR_WR;
	CLR_CS;
	SET_RD;

	PORTE = data1;
	PORTA = data2;

	SET_CS;
	SET_WR; 

	PORTE=0xFF;
	PORTA=0xFF;
}
  
//***************************************************************
void initialization()
{
	SET_RD;
	SET_WR;
	SET_CS;
	SET_CD;
	PORTA=0x00;
	PORTE=0x00;

	CLR_RESET;
	delay(200);
	SET_RESET;
	delay(500);

	wr_com8080_16(0x00,0xe2);	//PLL multiplier, set PLL clock to 120M
	wr_data8080_16(0x00,0x23);	//N=0x36 for 6.5M, 0x23 for 10M crystal
	wr_data8080_16(0x00,0x02);
	wr_data8080_16(0x00,0x04);

	wr_com8080_16(0x00,0xe0);	//PLL enable
	wr_data8080_16(0x00,0x01);
	delay(5);

	wr_com8080_16(0x00,0xe0);
	wr_data8080_16(0x00,0x03);
	delay(5);

	wr_com8080_16(0x00,0x01);
	delay(5);

	wr_com8080_16(0x00,0xe6);				//PLL setting for PCLK, depends on resolution
	wr_data8080_16(0x00,0x02);				//01  6.75M
	wr_data8080_16(0x00,0x47);				//8f
	wr_data8080_16(0x00,0xbe);				//00

	wr_com8080_16(0x00,0xB0);				//LCD SPECIFICATION
	wr_data8080_16(0x00,0x00);
	wr_data8080_16(0x00,0x00);
	wr_data8080_16(0x00,(HDP>>8)&0X00FF);	//Set HDP
	wr_data8080_16(0x00,(HDP&0X00FF));
	wr_data8080_16(0x00,((VDP>>8)&0X00FF));	//Set VDP
	wr_data8080_16(0x00,(VDP&0X00FF));
	wr_data8080_16(0x00,0x00);
	delay(5);

	wr_com8080_16(0x00,0xB4);				//HSYNC
	wr_data8080_16(0x00,((HT>>8)&0X00FF));	//Set HT
	wr_data8080_16(0x00,(HT&0X00FF));
	wr_data8080_16(0x00,(HPS>>8)&0X00FF);	//Set HPS
	wr_data8080_16(0x00,(HPS&0X00FF));
	wr_data8080_16(0x00,HPW);				//Set HPW
	wr_data8080_16(0x00,((LPS>>8)&0X00FF));	//SetLPS
	wr_data8080_16(0x00,(LPS&0X00FF));
	wr_data8080_16(0x00,0x00);

	wr_com8080_16(0x00,0xB6);				//VSYNC
	wr_data8080_16(0x00,((VT>>8)&0X00FF));	//Set VT
	wr_data8080_16(0x00,(VT&0X00FF));
	wr_data8080_16(0x00,((VPS>>8)&0X00FF));	//Set VPS
	wr_data8080_16(0x00,(VPS&0X00FF));
	wr_data8080_16(0x00,(VPW));				//Set VPW
	wr_data8080_16(0x00,((FPS>>8)&0X00FF));	//Set FPS
	wr_data8080_16(0x00,(FPS&0X00FF));

	wr_com8080_16(0x00,0x2A);
	wr_data8080_16(0x00,0);
	wr_data8080_16(0x00,0);
	wr_data8080_16(0x00,HDP>>8);
	wr_data8080_16(0x00,HDP&0x00ff);

	wr_com8080_16(0x00,0x2b);
	wr_data8080_16(0x00,0);
	wr_data8080_16(0x00,0);
	wr_data8080_16(0x00,VDP>>8);
	wr_data8080_16(0x00,VDP&0x00ff);

	wr_com8080_16(0x00,0x36);				//rotation
	wr_data8080_16(0x00,0x00);

	wr_com8080_16(0x00,0xf0);				//pixel data interface  
	wr_data8080_16(0x00,0x03);				//16-bit (565 format)

	/****************SET BL PMW*****************/

	/*wr_com8080_16(0x00,0xBE);//set PWM for B/L
	wr_data8080_16(0x00,0x06);
	wr_data8080_16(0x00,0xF0);
	wr_data8080_16(0x00,0x01);
	wr_data8080_16(0x00,0xF0);
	wr_data8080_16(0x00,0x00);
	wr_data8080_16(0x00,0x00);

	wr_com8080_16(0x00,0xBA);
	wr_data8080_16(0x00,0x00);*/

	/*************************************/

	wr_com8080_16(0x00,0x29);				//display on
}

void SendData(unsigned long color)
{
	write_data((color)>>16);	// color is red
	write_data((color)>>8);  	// color is green
	write_data(color);  		// color is blue
}





void WindowSet(unsigned int s_x,unsigned int e_x,unsigned int s_y,unsigned int e_y)
{
	write_command(0x2a);	//SET page address
	write_data((s_x)>>8);	//SET start page address=0
	write_data(s_x);
	write_data((e_x)>>8);	//SET end page address=319
	write_data(e_x);

	write_command(0x2b);	//SET column address
	write_data((s_y)>>8);	//SET start column address=0
	write_data(s_y);
	write_data((e_y)>>8);	//SET end column address=239
	write_data(e_y);
}


void FULL_ON(unsigned long dat)
{
	unsigned int x,y;
	WindowSet(0x0000,0x013f,0x0000,0x00ef);
	write_command(0x2c);
	for(x=0;x<240;x++)
	{
		for(y= 0;y<320;y++)
		{
			write_data(0x55);
			//SendData(dat);
		}
	}
}



void FRAME()
{
	unsigned int i,j;
	WindowSet(0x0000,0x013f,0x0000,0x00ef);
	write_command(0x2c);
	for(j= 0 ;j<40;j++)
	{
		for(i=0;i<8;i++)
		{
			SendData(0xffffff);	//white
		}
	}

	for(j=0;j<238;j++)
	{
		SendData(0xff0000);		//red
		for(i=0;i<159;i++)
		{
			SendData(0x000000);	//black
			SendData(0x000000);	//black
		}
	SendData(0x0000ff);			//blue
	}

 for(j= 0 ;j<40;j++)
  {
   for(i=0;i<8;i++)
   {
    SendData(0xffffff);			//white
   }
  }
}
//=========================3D==============

/*************************************************/
void Display_Home()
{
	write_command(0x004E);	// RAM address set
	write_data(0x0000);		// Page 58 of SSD2119 datasheet
	write_command(0x004F);	// RAM address set
	write_data(0x0000);		// Page 58 of SSD2119 datasheet

	write_command(0x0044);	// Vertical RAM address position
	write_data(0xEF00);		// Page 57 of SSD2119 datasheet
	write_command(0x0045);	// Horizontal RAM address position 
	write_data(0x0000);		// Page 57 of SSD2119 datasheet
	write_command(0x0046);	// Horizontal RAM address position
	write_data(0x013F);		// Page 57 of SSD2119 datasheet

	write_command(0x0022);	// RAM data write/read
}


void display_rgb(unsigned char data1,unsigned char data2)
{
	unsigned int  i,j;
	wr_com8080_16(0x00,0x2c);
	for(i=0;i<800;i++)
		for(j=0;j<480;j++)
		{
			wr_data8080_16(data1,data2);
		}
//	PORTA=0xFF;
//	PORTE=0xFF;
}


void Picture_Slide_Show()
{
	uint8 i;			//loop variable
	uint16 j;			//loop variable
	uint32 p;			//cluster
	uint16 *buffer;		//buffer
	uint16 pics = 1;
	uint16 slide_show_flag = 1;
	uint8 sector;
	uint32 pixels;

	if (totalfiles == 0)
	return;

	buffer = malloc(512);
	if (buffer == 0)
	return;

next: 	//label for "goto"
	display_rgb(BLACK>>8,BLACK);

	Search(PATH, &PictureInfo, &pics);                                         //find the file
	p = PictureInfo.deStartCluster + (((uint32)PictureInfo.deHighClust) << 16);//the first cluster of the file

	sector = 0;

	FAT_LoadPartCluster(p, sector, buffer);	//read a sector


	pixels = (uint32) 800 * 480; // total # of pixels to write

//	Display_Home();
//	write_command(0x22);

   wr_com8080_16(0x00,0x2c);
	j = 0; // byte count
	
	while (1)
	{
		wr_data8080_16(buffer[j]>>8,buffer[j]);  // write 16 bits
		pixels--;               // which is one pixel
		j++;                    // word count

		if (pixels == 0)        
		break;

		if (j == 256)           // time for a new sector     
		{
			sector++;
			if (sector == SectorsPerClust)
			{
				p = FAT_NextCluster(p);				// read next cluster
				sector = 0;
			}

			FAT_LoadPartCluster(p, sector, buffer);	// read a sector
			j = 0;
		}
	}

	if (slide_show_flag)
	{
		for (i=0;i<30;i++)	// delay for a while
		delay(0xffff);

		pics++;					// increment picture number
		if (pics > totalfiles)	// if last
		pics = 1;			// wrap around

		goto next;				// go show next pic
	}
	free(buffer);
}
/*************************************************/
void LCD_test()
{
	unsigned int i,j;
   wr_com8080_16(0x00,0x2c);


	for(i=0;i<480;i++)
	{
		for(j=0;j<800;j++)
		{
			if(i>419)wr_data8080_16(BLACK>>8,BLACK);
			else if(i>359)wr_data8080_16(BLUE>>8,BLUE);
			else if(i>299)wr_data8080_16(GREEN>>8,GREEN);
			else if(i>239)wr_data8080_16(CYAN>>8,CYAN);
			else if(i>179)wr_data8080_16(RED>>8,RED);
			else if(i>119)wr_data8080_16(MAGENTA>>8,MAGENTA);
			else if(i>59)wr_data8080_16(YELLOW>>8,YELLOW);
			else wr_data8080_16(WHITE>>8,WHITE);
		}
	}
}
/*************************************************/
void display_gray(void)//ʾҽ׻
{
    unsigned char j,k;

 wr_com8080_16(0x00,0x2c);    
	    
        for(k=0;k<17;k++)
        {
            for(j=0;j<480;j++)
		    {
            wr_data8080_16(0x00,0x00); //1

			 }
         } 
	    
        for(k=0;k<17;k++)
        {
            for(j=0;j<480;j++)
		    {
            wr_data8080_16(0x10,0x82); //2

			 }
         } 
	    
        for(k=0;k<17;k++)
        {
            for(j=0;j<480;j++)
		    {
            wr_data8080_16(0x21,0x04); //3

			 }
         } 
	    
        for(k=0;k<17;k++)
        {
            for(j=0;j<480;j++)
		    {
            wr_data8080_16(0x31,0x86); //4

			 }
         } 
	    
        for(k=0;k<17;k++)
        {
            for(j=0;j<480;j++)
		    {
            wr_data8080_16(0x42,0x08); //5

			 }
         } 
	    
        for(k=0;k<17;k++)
        {
            for(j=0;j<480;j++)
		    {
            wr_data8080_16(0x52,0x8a); //6

			 }
         } 
	    
        for(k=0;k<17;k++)
        {
            for(j=0;j<480;j++)
		    {
            wr_data8080_16(0x63,0x0c); //7

			 }
         } 
	    
        for(k=0;k<17;k++)
        {
            for(j=0;j<480;j++)
		    {
            wr_data8080_16(0x73,0x8e); //8

			 }
         } 
	    
        for(k=0;k<17;k++)
        {
            for(j=0;j<480;j++)
		    {
            wr_data8080_16(0x84,0x10); //9

			 }
         } 
	    
        for(k=0;k<17;k++)
        {
            for(j=0;j<480;j++)
		    {
            wr_data8080_16(0x94,0x92); //10

			 }
         } 
	    
        for(k=0;k<17;k++)
        {
            for(j=0;j<480;j++)
		    {
            wr_data8080_16(0xa5,0x14); //11

			 }
         } 
	    
        for(k=0;k<17;k++)
        {
            for(j=0;j<480;j++)
		    {
            wr_data8080_16(0xb5,0x96); //12

			 }
         } 
	    
        for(k=0;k<17;k++)
        {
            for(j=0;j<480;j++)
		    {
            wr_data8080_16(0xc6,0x18); //13

			 }
         } 
	    
        for(k=0;k<17;k++)
        {
            for(j=0;j<480;j++)
		    {
            wr_data8080_16(0xd6,0x9a); //14

			 }
         } 
	    
        for(k=0;k<17;k++)
        {
            for(j=0;j<480;j++)
		    {
            wr_data8080_16(0xe7,0x1c); //15

			 }
         } 
	    
        for(k=0;k<17;k++)
        {
            for(j=0;j<480;j++)
		    {
            wr_data8080_16(0xf7,0x9e); //16

			 }
         } 
	
            delay(8000);
}

/*************************************************/
int main()
{
	PORTD = 0xF8;	    // pull-up on switches, all others off
	DDRD  = 0x0F;   	// VPP and led pins output

	DDRA = 0xFF;    	// low data port, all outputs
	DDRE = 0xFF;    	// low data port, all outputs
	
	PORTC = 0xFE;	    // all pins high
	DDRC  = 0xFE;	    // all outputs


	uint8 retry = 0;

	OSCCAL = 0x00;    //in order to operate some low speed card the initialization should run at lowest speed

	delay(0xffff);

	MMC_SD_Init();    //SPI initialize

	// Initialize the display
//	initialization();

//	LCD_test();

	sd_restart: 	//label for "goto"

	while (MMC_SD_Reset())  // Initialize SD Card
	{
		retry++;
		if (retry > 254)
		{
			while (1)
			{
				goto sd_restart;				// try again
			}
		}
	}

	OSCCAL = 0xff;      //run at high speed for normal operation for SD Card

	delay(0xffff);      

	if (FAT_Init())   	//initialize file system - FAT16 and FAT32 are supported
	{
		while (1)
		{
			goto sd_restart;				// try again
		}
	}

	Search(PATH, &PictureInfo, &totalfiles);
//*/
int i;
	initialization();

/*

	Picture_Slide_Show();

	while (1)
	{
	}
	return 0;

*/


  while(1)
     { 
	LCD_test();
      
        display_rgb(0x00,0x00);
        delay(7000);
		
	display_rgb(0xff,0xff);
        delay(7000);


 //       display_gray();
	
	display_rgb(0xf8,0x00);
        delay(7000);
		
	display_rgb(0x07,0xe0);
        delay(7000);
		
	display_rgb(0x00,0x1f);
        delay(7000);

	Picture_Slide_Show();

}



}
/*************************************************/
